@@ -15,15 +15,17 @@
 
 import argparse
 from datetime import datetime
-import functools
 import json
 import os
 import ssl
 import sys
 
 import flask
+from ironic_lib import auth_basic
 from werkzeug import exceptions as wz_exc
 
+from sushy_tools.emulator import api_utils
+from sushy_tools.emulator.controllers import virtual_media as vmctl
 from sushy_tools.emulator import memoize
 from sushy_tools.emulator.resources import chassis as chsdriver
 from sushy_tools.emulator.resources import drives as drvdriver
@@ -35,18 +37,56 @@ from sushy_tools.emulator.resources.systems import novadriver
 from sushy_tools.emulator.resources import vmedia as vmddriver
 from sushy_tools.emulator.resources import volumes as voldriver
 from sushy_tools import error
-from sushy_tools.error import FishyError
+
+
+def _render_error(message):
+    return {
+        "error": {
+            "code": "Base.1.0.GeneralError",
+            "message": message,
+            "@Message.ExtendedInfo": [
+                {
+                    "@odata.type": ("/redfish/v1/$metadata"
+                                    "#Message.1.0.0.Message"),
+                    "MessageId": "Base.1.0.GeneralError"
+                }
+            ]
+        }
+    }
+
+
+class RedfishAuthMiddleware(auth_basic.BasicAuthMiddleware):
+
+    _EXCLUDE_PATHS = frozenset(['', 'redfish', 'redfish/v1'])
+
+    def __call__(self, env, start_response):
+        path = env.get('PATH_INFO', '')
+        if path.strip('/') in self._EXCLUDE_PATHS:
+            return self.app(env, start_response)
+        else:
+            return super().__call__(env, start_response)
+
+    def format_exception(self, e):
+        response = super().format_exception(e)
+        response.json_body = _render_error(str(e))
+        return response
 
 
 class Application(flask.Flask):
 
-    def __init__(self):
+    def __init__(self, extra_config=None):
         super().__init__(__name__)
         # Turn off strict_slashes on all routes
         self.url_map.strict_slashes = False
         config_file = os.environ.get('SUSHY_EMULATOR_CONFIG')
         if config_file:
             self.config.from_pyfile(config_file)
+        if extra_config:
+            self.config.update(extra_config)
+
+        auth_file = self.config.get("SUSHY_EMULATOR_AUTH_FILE")
+        if auth_file:
+            self.wsgi_app = RedfishAuthMiddleware(self.wsgi_app, auth_file)
 
     @property
     @memoize.memoize()
@@ -113,60 +153,17 @@ class Application(flask.Flask):
 
 
 app = Application()
-
-
-def instance_denied(**kwargs):
-    deny = True
-
-    try:
-        deny = (kwargs['identity'] not in
-                app.config['SUSHY_EMULATOR_ALLOWED_INSTANCES'])
-
-    except KeyError:
-        deny = False
-
-    finally:
-        if deny:
-            app.logger.warning('Instance %s access denied',
-                               kwargs.get('identity'))
-
-        return deny
-
-
-def ensure_instance_access(decorated_func):
-    @functools.wraps(decorated_func)
-    def decorator(*args, **kwargs):
-        if instance_denied(**kwargs):
-            raise FishyError('Error finding instance')
-
-        return decorated_func(*args, **kwargs)
-
-    return decorator
-
-
-def returns_json(decorated_func):
-    @functools.wraps(decorated_func)
-    def decorator(*args, **kwargs):
-        response = decorated_func(*args, **kwargs)
-        if isinstance(response, flask.Response):
-            return response
-        if isinstance(response, tuple):
-            contents, status = response
-        else:
-            contents, status = response, 200
-        return flask.Response(response=contents, status=status,
-                              content_type='application/json')
-
-    return decorator
+app.register_blueprint(vmctl.virtual_media)
 
 
 @app.errorhandler(Exception)
-@returns_json
+@api_utils.returns_json
 def all_exception_handler(message):
     if isinstance(message, error.AliasAccessError):
         url = flask.url_for(flask.request.endpoint, identity=message.args[0])
         return flask.redirect(url, code=307, Response=flask.Response)
 
+    code = getattr(message, 'code', 500)
     if (isinstance(message, error.FishyError)
             or isinstance(message, wz_exc.HTTPException)):
         app.logger.debug(
@@ -175,17 +172,17 @@ def all_exception_handler(message):
         app.logger.exception(
             'Unexpected %s: %s', message.__class__.__name__, message)
 
-    return flask.render_template('error.json', message=message), 500
+    return flask.render_template('error.json', message=message), code
 
 
 @app.route('/redfish/v1/')
-@returns_json
+@api_utils.returns_json
 def root_resource():
     return flask.render_template('root.json')
 
 
 @app.route('/redfish/v1/Chassis')
-@returns_json
+@api_utils.returns_json
 def chassis_collection_resource():
     app.logger.debug('Serving chassis list')
 
@@ -196,7 +193,7 @@ def chassis_collection_resource():
 
 
 @app.route('/redfish/v1/Chassis/<identity>', methods=['GET', 'PATCH'])
-@returns_json
+@api_utils.returns_json
 def chassis_resource(identity):
     chassis = app.chassis
 
@@ -248,7 +245,7 @@ def chassis_resource(identity):
 
 
 @app.route('/redfish/v1/Chassis/<identity>/Thermal', methods=['GET'])
-@returns_json
+@api_utils.returns_json
 def thermal_resource(identity):
     chassis = app.chassis
 
@@ -272,7 +269,7 @@ def thermal_resource(identity):
 
 
 @app.route('/redfish/v1/Managers')
-@returns_json
+@api_utils.returns_json
 def manager_collection_resource():
     app.logger.debug('Serving managers list')
 
@@ -295,15 +292,11 @@ def jsonify(obj_type, obj_version, obj):
 
 
 @app.route('/redfish/v1/Managers/<identity>', methods=['GET'])
-@returns_json
+@api_utils.returns_json
 def manager_resource(identity):
     app.logger.debug('Serving resources for manager "%s"', identity)
 
-    try:
-        manager = app.managers.get_manager(identity)
-    except error.FishyError as exc:
-        return str(exc), 404
-
+    manager = app.managers.get_manager(identity)
     systems = app.managers.get_managed_systems(manager)
     chassis = app.managers.get_managed_chassis(manager)
 
@@ -345,129 +338,11 @@ def manager_resource(identity):
     })
 
 
-@app.route('/redfish/v1/Managers/<identity>/VirtualMedia', methods=['GET'])
-@returns_json
-def virtual_media_collection_resource(identity):
-    app.logger.debug('Serving virtual media resources for '
-                     'manager "%s"', identity)
-
-    return flask.render_template(
-        'virtual_media_collection.json',
-        identity=identity,
-        uuid=app.managers.get_manager(identity)['UUID'],
-        devices=app.vmedia.devices
-    )
-
-
-@app.route('/redfish/v1/Managers/<identity>/VirtualMedia/<device>',
-           methods=['GET'])
-@returns_json
-def virtual_media_resource(identity, device):
-    try:
-        device_name = app.vmedia.get_device_name(
-            identity, device)
-
-        media_types = app.vmedia.get_device_media_types(
-            identity, device)
-
-        (image_name, image_url, inserted,
-         write_protected) = app.vmedia.get_device_image_info(
-            identity, device)
-
-    except error.FishyError as ex:
-        app.logger.warning(
-            'Virtual media %s at manager %s error: '
-            '%s', device, identity, ex)
-        return 'Not found', 404
-
-    app.logger.debug('Serving virtual media %s at '
-                     'manager "%s"', device, identity)
-
-    return flask.render_template(
-        'virtual_media.json',
-        identity=identity,
-        device=device,
-        name=device_name,
-        media_types=media_types,
-        image_url=image_url,
-        image_name=image_name,
-        inserted=inserted,
-        write_protected=write_protected
-    )
-
-
-@app.route('/redfish/v1/Managers/<identity>/VirtualMedia/<device>'
-           '/Actions/VirtualMedia.InsertMedia',
-           methods=['POST'])
-@returns_json
-def virtual_media_insert(identity, device):
-    image = flask.request.json.get('Image')
-    inserted = flask.request.json.get('Inserted', True)
-    write_protected = flask.request.json.get('WriteProtected', False)
-
-    manager = app.managers.get_manager(identity)
-    systems = app.managers.get_managed_systems(manager)
-    if not systems:
-        app.logger.warning('Manager %s manages no systems', identity)
-        return '', 204
-
-    image_path = app.vmedia.insert_image(
-        identity, device, image, inserted, write_protected)
-
-    for system in systems:
-        try:
-            app.systems.set_boot_image(
-                system, device, boot_image=image_path,
-                write_protected=write_protected)
-
-        except error.NotSupportedError as ex:
-            app.logger.warning(
-                'System %s failed to set boot image %s on device %s: '
-                '%s', system, image_path, device, ex)
-
-    app.logger.info(
-        'Virtual media placed into device %(dev)s of manager %(mgr)s for '
-        'systems %(sys)s. Image %(img)s inserted %(ins)s',
-        {'dev': device, 'mgr': identity, 'sys': systems,
-         'img': image or '<empty>', 'ins': inserted})
-
-    return '', 204
-
-
-@app.route('/redfish/v1/Managers/<identity>/VirtualMedia/<device>'
-           '/Actions/VirtualMedia.EjectMedia',
-           methods=['POST'])
-@returns_json
-def virtual_media_eject(identity, device):
-    app.vmedia.eject_image(identity, device)
-
-    manager = app.managers.get_manager(identity)
-    systems = app.managers.get_managed_systems(manager)
-    if not systems:
-        app.logger.warning('Manager %s manages no systems', identity)
-        return '', 204
-
-    for system in systems:
-        try:
-            app.systems.set_boot_image(system, device)
-
-        except error.NotSupportedError as ex:
-            app.logger.warning(
-                'System %s failed to remove boot image from device %s: '
-                '%s', system, device, ex)
-
-    app.logger.info(
-        'Virtual media ejected from device %s manager %s systems %s',
-        device, identity, systems)
-
-    return '', 204
-
-
 @app.route('/redfish/v1/Systems')
-@returns_json
+@api_utils.returns_json
 def system_collection_resource():
     systems = [system for system in app.systems.systems
-               if not instance_denied(identity=system)]
+               if not api_utils.instance_denied(identity=system)]
 
     app.logger.debug('Serving systems list')
 
@@ -476,8 +351,8 @@ def system_collection_resource():
 
 
 @app.route('/redfish/v1/Systems/<identity>', methods=['GET', 'PATCH'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def system_resource(identity):
     if flask.request.method == 'GET':
 
@@ -543,8 +418,8 @@ def system_resource(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/EthernetInterfaces',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def ethernet_interfaces_collection(identity):
     nics = app.systems.get_nics(identity)
 
@@ -555,8 +430,8 @@ def ethernet_interfaces_collection(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/EthernetInterfaces/<nic_id>',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def ethernet_interface(identity, nic_id):
     nics = app.systems.get_nics(identity)
 
@@ -565,13 +440,13 @@ def ethernet_interface(identity, nic_id):
             return flask.render_template(
                 'ethernet_interface.json', identity=identity, nic=nic)
 
-    return 'Not found', 404
+    raise error.NotFound()
 
 
 @app.route('/redfish/v1/Systems/<identity>/Processors',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def processors_collection(identity):
     processors = app.systems.get_processors(identity)
 
@@ -582,8 +457,8 @@ def processors_collection(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/Processors/<processor_id>',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def processor(identity, processor_id):
     processors = app.systems.get_processors(identity)
 
@@ -592,13 +467,13 @@ def processor(identity, processor_id):
             return flask.render_template(
                 'processor.json', identity=identity, processor=proc)
 
-    return 'Not found', 404
+    raise error.NotFound()
 
 
 @app.route('/redfish/v1/Systems/<identity>/Actions/ComputerSystem.Reset',
            methods=['POST'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def system_reset_action(identity):
     reset_type = flask.request.json.get('ResetType')
 
@@ -611,8 +486,8 @@ def system_reset_action(identity):
 
 
 @app.route('/redfish/v1/Systems/<identity>/BIOS', methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def bios(identity):
     bios = app.systems.get_bios(identity)
 
@@ -626,8 +501,8 @@ def bios(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/BIOS/Settings',
            methods=['GET', 'PATCH'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def bios_settings(identity):
 
     if flask.request.method == 'GET':
@@ -652,8 +527,8 @@ def bios_settings(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/BIOS/Actions/Bios.ResetBios',
            methods=['POST'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def system_reset_bios(identity):
     app.systems.reset_bios(identity)
 
@@ -664,8 +539,8 @@ def system_reset_bios(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/SimpleStorage',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def simple_storage_collection(identity):
     simple_storage_controllers = (
         app.systems.get_simple_storage_collection(identity))
@@ -677,8 +552,8 @@ def simple_storage_collection(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/SimpleStorage/<simple_storage_id>',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def simple_storage(identity, simple_storage_id):
     simple_storage_controllers = (
         app.systems.get_simple_storage_collection(identity))
@@ -686,15 +561,15 @@ def simple_storage(identity, simple_storage_id):
         storage_controller = simple_storage_controllers[simple_storage_id]
     except KeyError:
         app.logger.debug('"%s" Simple Storage resource was not found')
-        return 'Not found', 404
+        raise error.NotFound()
     return flask.render_template('simple_storage.json', identity=identity,
                                  simple_storage=storage_controller)
 
 
 @app.route('/redfish/v1/Systems/<identity>/Storage',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def storage_collection(identity):
     uuid = app.systems.uuid(identity)
 
@@ -707,8 +582,8 @@ def storage_collection(identity):
 
 @app.route('/redfish/v1/Systems/<identity>/Storage/<storage_id>',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def storage(identity, storage_id):
     uuid = app.systems.uuid(identity)
     storage_col = app.storage.get_storage_col(uuid)
@@ -718,13 +593,13 @@ def storage(identity, storage_id):
             return flask.render_template(
                 'storage.json', identity=identity, storage=stg)
 
-    return 'Not found', 404
+    raise error.NotFound()
 
 
 @app.route('/redfish/v1/Systems/<identity>/Storage/<stg_id>/Drives/<drv_id>',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def drive_resource(identity, stg_id, drv_id):
     uuid = app.systems.uuid(identity)
     drives = app.drives.get_drives(uuid, stg_id)
@@ -734,13 +609,13 @@ def drive_resource(identity, stg_id, drv_id):
             return flask.render_template(
                 'drive.json', identity=identity, storage_id=stg_id, drive=drv)
 
-    return 'Not found', 404
+    raise error.NotFound()
 
 
 @app.route('/redfish/v1/Systems/<identity>/Storage/<storage_id>/Volumes',
            methods=['GET', 'POST'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def volumes_collection(identity, storage_id):
     uuid = app.systems.uuid(identity)
 
@@ -781,8 +656,8 @@ def volumes_collection(identity, storage_id):
 
 @app.route('/redfish/v1/Systems/<identity>/Storage/<stg_id>/Volumes/<vol_id>',
            methods=['GET'])
-@ensure_instance_access
-@returns_json
+@api_utils.ensure_instance_access
+@api_utils.returns_json
 def volume(identity, stg_id, vol_id):
     uuid = app.systems.uuid(identity)
     vol_col = app.volumes.get_volumes_col(uuid, stg_id)
@@ -797,11 +672,11 @@ def volume(identity, stg_id, vol_id):
                     'volume.json', identity=identity, storage_id=stg_id,
                     volume=vol)
 
-    return 'Not Found', 404
+    raise error.NotFound()
 
 
 @app.route('/redfish/v1/Registries')
-@returns_json
+@api_utils.returns_json
 def registry_file_collection():
     app.logger.debug('Serving registry file collection')
 
@@ -810,7 +685,7 @@ def registry_file_collection():
 
 
 @app.route('/redfish/v1/Registries/BiosAttributeRegistry.v1_0_0')
-@returns_json
+@api_utils.returns_json
 def bios_attribute_registry_file():
     app.logger.debug('Serving BIOS attribute registry file')
 
@@ -819,7 +694,7 @@ def bios_attribute_registry_file():
 
 
 @app.route('/redfish/v1/Registries/Messages')
-@returns_json
+@api_utils.returns_json
 def message_registry_file():
     app.logger.debug('Serving message registry file')
 
@@ -827,16 +702,16 @@ def message_registry_file():
         'message_registry_file.json')
 
 
-@app.route('/redfish/v1/Systems/<identity>/Bios/BiosRegistry')
-@returns_json
-def bios_registry(identity):
+@app.route('/redfish/v1/Systems/Bios/BiosRegistry')
+@api_utils.returns_json
+def bios_registry():
     app.logger.debug('Serving BIOS registry')
 
-    return flask.render_template('bios_registry.json', identity=identity)
+    return flask.render_template('bios_registry.json')
 
 
 @app.route('/redfish/v1/Registries/Messages/Registry')
-@returns_json
+@api_utils.returns_json
 def message_registry():
     app.logger.debug('Serving message registry')
 
