@@ -126,10 +126,14 @@ class LibvirtDriver(AbstractSystemsDriver):
 
     DEFAULT_BIOS_ATTRIBUTES = {"BootMode": "Uefi",
                                "EmbeddedSata": "Raid",
+                               "L2Cache": "10x256 KB",
                                "NicBoot1": "NetworkBoot",
+                               "NumCores": "10",
+                               "QuietBoot": "true",
                                "ProcTurboMode": "Enabled",
                                "SecureBootStatus": "Enabled",
-                               "SerialNumber": "QPX12345"}
+                               "SerialNumber": "QPX12345",
+                               "SysPassword": ""}
 
     STORAGE_POOL = 'default'
 
@@ -181,7 +185,7 @@ class LibvirtDriver(AbstractSystemsDriver):
 
                     self._logger.debug(msg)
 
-                    raise error.FishyError(msg)
+                    raise error.NotFound(msg)
 
             raise error.AliasAccessError(domain.UUIDString())
 
@@ -209,7 +213,7 @@ class LibvirtDriver(AbstractSystemsDriver):
         in place of system name if there are duplicates.
 
         :param identity: libvirt domain name or UUID
-
+        :raises: NotFound if the system cannot be found
         :returns: computer system UUID
         """
         domain = self._get_domain(identity, readonly=True)
@@ -219,7 +223,7 @@ class LibvirtDriver(AbstractSystemsDriver):
         """Get computer system name by name
 
         :param identity: libvirt domain name or UUID
-
+        :raises: NotFound if the system cannot be found
         :returns: computer system name
         """
         domain = self._get_domain(identity, readonly=True)
@@ -497,7 +501,7 @@ class LibvirtDriver(AbstractSystemsDriver):
             msg = ('Unknown boot mode requested: '
                    '%(boot_mode)s' % {'boot_mode': boot_mode})
 
-            raise error.FishyError(msg)
+            raise error.BadRequest(msg)
 
         os_elements = tree.findall('os')
         if len(os_elements) != 1:
@@ -935,7 +939,7 @@ class LibvirtDriver(AbstractSystemsDriver):
                 lv_device = self.BOOT_DEVICE_MAP[device]
 
             except KeyError:
-                raise error.FishyError(
+                raise error.BadRequest(
                     'Unknown device %s at %s' % (device, identity))
 
             disk_elements = device_element.findall('disk')
@@ -1022,7 +1026,7 @@ class LibvirtDriver(AbstractSystemsDriver):
             lv_device = self.BOOT_DEVICE_MAP[device]
 
         except KeyError:
-            raise error.FishyError(
+            raise error.BadRequest(
                 'Unknown device %s at %s' % (device, identity))
 
         device_element = domain_tree.find('devices')
