@@ -9,6 +9,8 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
+import tempfile
 from unittest import mock
 
 from oslotest import base
@@ -51,6 +53,37 @@ class CommonTestCase(EmulatorTestCase):
         self.assertEqual('RedvirtService', response.json['Id'])
 
 
+TEST_PASSWD = \
+    b"admin:$2y$05$mYl8KMwM94l4LR/sw1teIeA6P2u8gfX16e8wvT7NmGgAM5r9jgLl."
+
+
+class AuthenticatedTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.auth_file = tempfile.NamedTemporaryFile()
+        self.auth_file.write(TEST_PASSWD)
+        self.auth_file.flush()
+        self.addCleanup(self.auth_file.close)
+        app = main.Application({
+            'SUSHY_EMULATOR_AUTH_FILE': self.auth_file.name})
+        self.app = app.test_client()
+
+    def test_root_resource(self):
+        response = self.app.get('/redfish/v1/')
+        # 404 because this application does not have any routes
+        self.assertEqual(404, response.status_code, response.data)
+
+    def test_authenticated_resource(self):
+        response = self.app.get('/redfish/v1/Systems/',
+                                auth=('admin', 'password'))
+        self.assertEqual(404, response.status_code, response.data)
+
+    def test_authentication_failed(self):
+        response = self.app.get('/redfish/v1/Systems/')
+        self.assertEqual(401, response.status_code, response.data)
+
+
 class ChassisTestCase(EmulatorTestCase):
 
     @patch_resource('chassis')
@@ -277,33 +310,6 @@ class SystemsTestCase(EmulatorTestCase):
         self.assertEqual(500, response.status_code)
 
 
-class InstanceDeniedTestCase(EmulatorTestCase):
-
-    @mock.patch.dict(main.app.config, {}, clear=True)
-    def test_instance_denied_allow_all(self):
-        self.assertFalse(main.instance_denied(identity='x'))
-
-    @mock.patch.dict(
-        main.app.config, {'SUSHY_EMULATOR_ALLOWED_INSTANCES': {}})
-    def test_instance_denied_disallow_all(self):
-        self.assertTrue(main.instance_denied(identity='a'))
-
-    def test_instance_denied_undefined_option(self):
-        with mock.patch.dict(main.app.config):
-            main.app.config.pop('SUSHY_EMULATOR_ALLOWED_INSTANCES', None)
-            self.assertFalse(main.instance_denied(identity='a'))
-
-    @mock.patch.dict(
-        main.app.config, {'SUSHY_EMULATOR_ALLOWED_INSTANCES': {'a'}})
-    def test_instance_denied_allow_some(self):
-        self.assertFalse(main.instance_denied(identity='a'))
-
-    @mock.patch.dict(
-        main.app.config, {'SUSHY_EMULATOR_ALLOWED_INSTANCES': {'a'}})
-    def test_instance_denied_disallow_some(self):
-        self.assertTrue(main.instance_denied(identity='b'))
-
-
 @patch_resource('systems')
 class BiosTestCase(EmulatorTestCase):
 
@@ -421,87 +427,6 @@ class EthernetInterfacesTestCase(EmulatorTestCase):
         self.assertEqual(404, response.status_code)
 
 
-@patch_resource('vmedia')
-@patch_resource('managers')
-class VirtualMediaTestCase(EmulatorTestCase):
-
-    def test_virtual_media_collection(self, managers_mock, vmedia_mock):
-        managers_mock = managers_mock.return_value
-        managers_mock.managers = [self.uuid]
-        managers_mock.get_manager.return_value = {'UUID': self.uuid}
-        vmedia_mock.return_value.devices = ['CD', 'Floppy']
-
-        response = self.app.get(
-            'redfish/v1/Managers/%s/VirtualMedia' % self.uuid)
-
-        self.assertEqual(200, response.status_code)
-        self.assertEqual('Virtual Media Services', response.json['Name'])
-        self.assertEqual(2, response.json['Members@odata.count'])
-        self.assertEqual(
-            ['/redfish/v1/Managers/%s/VirtualMedia/CD' % self.uuid,
-             '/redfish/v1/Managers/%s/VirtualMedia/Floppy' % self.uuid],
-            [m['@odata.id'] for m in response.json['Members']])
-
-    def test_virtual_media_collection_empty(self, managers_mock, vmedia_mock):
-        vmedia_mock.return_value.get_devices.return_value = []
-
-        response = self.app.get(
-            'redfish/v1/Managers/' + self.uuid + '/VirtualMedia')
-
-        self.assertEqual(200, response.status_code)
-        self.assertEqual('Virtual Media Services', response.json['Name'])
-        self.assertEqual(0, response.json['Members@odata.count'])
-        self.assertEqual([], response.json['Members'])
-
-    def test_virtual_media(self, managers_mock, vmedia_mock):
-        vmedia_mock = vmedia_mock.return_value
-        vmedia_mock.get_device_name.return_value = 'CD'
-        vmedia_mock.get_device_media_types.return_value = [
-            'CD', 'DVD']
-        vmedia_mock.get_device_image_info.return_value = [
-            'image-of-a-fish', 'fishy.iso', True, True]
-
-        response = self.app.get(
-            '/redfish/v1/Managers/%s/VirtualMedia/CD' % self.uuid)
-
-        self.assertEqual(200, response.status_code)
-        self.assertEqual('CD', response.json['Id'])
-        self.assertEqual(['CD', 'DVD'], response.json['MediaTypes'])
-        self.assertEqual('fishy.iso', response.json['Image'])
-        self.assertEqual('image-of-a-fish', response.json['ImageName'])
-        self.assertTrue(response.json['Inserted'])
-        self.assertTrue(response.json['WriteProtected'])
-
-    def test_virtual_media_not_found(self, managers_mock, vmedia_mock):
-        vmedia_mock.return_value.get_device_name.side_effect = error.FishyError
-
-        response = self.app.get(
-            '/redfish/v1/Managers/%s/VirtualMedia/DVD-ROM' % self.uuid)
-
-        self.assertEqual(404, response.status_code)
-
-    def test_virtual_media_insert(self, managers_mock, vmedia_mock):
-        response = self.app.post(
-            '/redfish/v1/Managers/%s/VirtualMedia/CD/Actions/'
-            'VirtualMedia.InsertMedia' % self.uuid,
-            json={"Image": "http://fish.iso"})
-
-        self.assertEqual(204, response.status_code)
-
-        vmedia_mock.return_value.insert_image.called_once_with(
-            'CD', 'http://fish.iso', True, False)
-
-    def test_virtual_media_eject(self, managers_mock, vmedia_mock):
-        response = self.app.post(
-            '/redfish/v1/Managers/%s/VirtualMedia/CD/Actions/'
-            'VirtualMedia.EjectMedia' % self.uuid,
-            json={})
-
-        self.assertEqual(204, response.status_code)
-
-        vmedia_mock.return_value.eject_image.called_once_with('CD')
-
-
 @patch_resource('systems')
 class StorageTestCase(EmulatorTestCase):
 
@@ -810,17 +735,16 @@ class RegistryTestCase(EmulatorTestCase):
 
     def test_bios_registry(self):
         response = self.app.get(
-            '/redfish/v1/Systems/%s/Bios/BiosRegistry' % self.uuid)
+            '/redfish/v1/Systems/Bios/BiosRegistry')
         self.assertEqual(200, response.status_code)
-        self.assertEqual('/redfish/v1/Systems/%s/Bios/BiosRegistry'
-                         % self.uuid,
+        self.assertEqual('/redfish/v1/Systems/Bios/BiosRegistry',
                          response.json['@odata.id'])
         self.assertEqual('BIOS Attribute Registry',
                          response.json['Name'])
         self.assertEqual('BiosAttributeRegistryP89.v1_0_0',
                          response.json['Id'])
         entries = response.json['RegistryEntries']
-        self.assertEqual(len(entries['Attributes']), 6)
+        self.assertEqual(len(entries['Attributes']), 10)
         self.assertEqual({'AttributeName': 'ProcTurboMode',
                           'CurrentValue': None,
                           'DisplayName': 'Turbo Boost',
